from importlib import import_module

from django.core.exceptions import ImproperlyConfigured


def get_module_class(class_path):
    """
    imports and returns module class from ``path.to.module.Class``
    argument
    """
    mod_name, cls_name = class_path.rsplit('.', 1)

    try:
        mod = import_module(mod_name)
    except ImportError as ex:
        raise ImproperlyConfigured('Error importing module %s: "%s"' % (mod_name, ex))

    return getattr(mod, cls_name)



import json
from optparse import make_option

import django

from evostream.commands import list_streams
from evostream.management.base import EvoStreamCommand


class Command(EvoStreamCommand):
    help = 'Provides a detailed description of all active streams.'

    requires_system_checks = False

    if django.VERSION[:2] > (1, 7):
        def add_arguments(self, parser):
            parser.add_argument('--disable-internal-streams', action='store',
                                type=int, choices=[0, 1], default=0,
                                dest='disableInternalStreams',
                                help='Filtering out internal streams from the list')
    else:
        option_list = EvoStreamCommand.option_list + (
            make_option('--disable-internal-streams', action='store',
                        type='choice', choices=['0', '1'], default='0',
                        dest='disableInternalStreams',
                        help='Filtering out internal streams from the list'),
        )

    def get_results(self, **options):
        return list_streams(**options)

    def format_results(self, results, verbosity):
        if verbosity > 1:
            super(Command, self).format_results(results, verbosity=verbosity)
        else:
            for stream in results:
                for key, val in stream.items():
                    if key in ('uniqueId', 'name'):
                        self.stdout.write(key + ': ' + json.dumps(val) + '\n')
        self.stdout.write('\n')



from evostream.commands import list_config
from evostream.management.base import EvoStreamCommand


class Command(EvoStreamCommand):
    help = 'Returns a list with all push/pull configurations.'

    requires_system_checks = False

    def get_results(self, **options):
        return list_config()



from functools import wraps
import logging

from .default import protocol


execute = protocol.execute
logger = logging.getLogger(__name__)


def expected(*expected_keys):
    expected_keys = set(expected_keys)

    def command_decorator(func):
        def wrapped_func(*args, **kwargs):
            unexpected = set(kwargs.keys()) - expected_keys

            if bool(unexpected):
                unexpected = ', '.join([key for key in list(unexpected)])
                logger.warning('Function %s: Unexpected argument(s): %s', func.__name__, unexpected)

            kwargs = dict((key, val) for key, val in kwargs.items()
                          if key in expected_keys)

            return func(*args, **kwargs)
        return wraps(func)(wrapped_func)
    return command_decorator


@expected('keepAlive', 'localStreamName', 'forceTcp', 'tcUrl', 'pageUrl',
          'swfUrl', 'ttl', 'tos', 'rtcpDetectionInterval', 'emulateUserAgent',
          'isAudio', 'audioCodecBytes', 'spsBytes', 'ppsBytes', 'ssmIp')
def pull_stream(uri, **kwargs):
    """
    This will try to pull in a stream from an external source. Once a stream
    has been successfully pulled it is assigned a 'local stream name' which can
    be used to access the stream from the EMS.

    :param uri: The URI of the external stream. Can be RTMP, RTSP or
        unicast/multicast (d) mpegts
    :type uri: str

    :param keepAlive: If keepAlive is set to 1, the server will attempt to
        reestablish connection with astream source after a connection has been
        lost. The reconnect will be attempted once every second
        (default: 1 true)
    :type keepAlive: int

    :param localStreamName: If provided, the stream will be given this
        name. Otherwise, a fallback techniqueis used to determine the stream
        name (based on the URI)
    :type localStreamName: str

    :param forceTcp: If 1 and if the stream is RTSP, a TCP connection will
        be forced. Otherwise the transport mechanism will be negotiated (UDP
        or TCP) (default: 1 true)
    :type forceTcp: int

    :param tcUrl: When specified, this value will be used to set the TC URL in
        the initial RTMPconnect invoke
    :type tcUrl: str

    :param pageUrl: When specified, this value will be used to set the
        originating web page address inthe initial RTMP connect invoke
    :type pageUrl: str

    :param swfUrl: When specified, this value will be used to set the
        originating swf URL in theinitial RTMP connect invoke
    :type swfUrl: str

    :param rangeStart: For RTSP and RTMP connections.  A value fromwhich the
        playback should start expressed in seconds. There are 2 specialvalues:
        -2 and -1. For more information, please read about start/len
        parameters here:
        http://livedocs.adobe.com/flashmediaserver/3.0/hpdocs/help.html?content=00000185.html
    :type rangeStart: int

    :param rangeEnd: The length in seconds for the playback. -1 is a special
        value. For more information, please read about start/len parameters
        here:
        http://livedocs.adobe.com/flashmediaserver/3.0/hpdocs/help.html?content=00000185.html
    :type rangeEnd: int

    :param ttl: Sets the IP_TTL (time to live) option on the socket
    :type ttl: int

    :param tos: Sets the IP_TOS (Type of Service) option on the socket
    :type tos: int

    :param rtcpDetectionInterval: How much time (in seconds) should the server
        wait for RTCP packets before declaring the RTSP stream as a RTCP-less
        stream
    :type rtcpDetectionInterval: int

    :param emulateUserAgent: When specified, this value will be used as the
        user agent string. It is meaningful only for RTMP
    :type emulateUserAgent: str

    :param isAudio: If 1 and if the stream is RTP, it indicates that the
        currently pulled stream is an audio source. Otherwise the pulled
        source is assumed as a video source
    :type isAudio: int

    :param audioCodecBytes: The audio codec setup of this RTP stream if it is
        audio. Represented as hex format without '0x' or 'h'. For example:
        audioCodecBytes=1190
    :type audioCodecBytes: str

    :param spsBytes: The video SPS bytes of this RTP stream if it is video. It
        should be base 64 encoded.
    :type spsBytes: str

    :param ppsBytes: The video PPS bytes of this RTP stream if it is video. It
        should be base 64 encoded
    :type ppsBytes: str

    :param ssmIp: The source IP from source-specific-multicast. Only usable
        when doing UDP based pull
    :type ssmIp: str

    :param httpProxy: This parameter has two valid values: IP:Port - This
        value combination specifies an RTSP HTTP Proxy from which the RTSP
        stream should be pulled from Self - Specifying "self" as the value
        implies pulling RTSP over HTTP
    :type httpProxy: str

    :link: http://docs.evostream.com/ems_api_definition/pullstream
    """
    return execute('pullstream', uri=uri, **kwargs)


@expected('id', 'localStreamName')
def get_stream_info(**kwargs):
    """
    Returns a detailed set of information about a stream.

    :param id: The uniqueId of the stream. Usually a value returned by
        listStreamsIDs. This parameter is not mandatory but either this or the
        localStreamName should be present to identify the particular stream
    :type id: int

    :param localStreamName: The name of the stream. This parameter is not
        mandatory but either this or the id should be present to identify the
        particular stream
    :type localStreamName: str

    :link: http://docs.evostream.com/ems_api_definition/getstreaminfo
    """
    return execute('getstreaminfo', **kwargs)


@expected('disableInternalStreams')
def list_streams(**kwargs):
    """
    Provides a detailed description of all active streams.

    :param disableInternalStreams: If this is 1 (true), internal streams
        (origin-edge related) are filtered out from the list
    :type disableInternalStreams: int

    :link: http://docs.evostream.com/ems_api_definition/liststreams
    """
    return execute('liststreams', **kwargs)


@expected('id', 'localStreamName', 'permanently')
def shutdown_stream(**kwargs):
    """
    Terminates a specific stream. When permanently=1 is used, this command is
    analogous to removeConfig.

    :param id: The uniqueId of the stream that needs to be terminated. The
        stream ID's can be obtained using the listStreams command. This
        parameter is not mandatory but either this or localStreamName should be
        present to identify the particular stream
    :type id: int

    :param localStreamName: The name of the inbound stream which you wish to
        terminate. This will also terminate any outbound streams that are
        dependent upon this input stream. This parameter is not mandatory but
        either this or the id should be present to identify the particular
        stream
    :type localStreamName: str

    :param permanently: If true, the corresponding push/pull configuration will
        also be terminated. Therefore, the stream will NOT be reconnected when
        the server restarts
    :type permanently: int

    :link: http://docs.evostream.com/ems_api_definition/shutdownstream
    """
    return execute('shutdownstream', **kwargs)


def list_config():
    """
    Returns a list with all push/pull configurations.

    Whenever the pullStream or pushStream interfaces are called, a record
    containing the details of the pull or push is created in the
    pullpushconfig.xml file. Then, the next time the EMS is started, the
    pullpushconfig.xml file is read, and the EMS attempts to reconnect all of
    the previous pulled or pushed streams.

    :link: http://docs.evostream.com/ems_api_definition/listconfig
    """
    return execute('listconfig')


@expected('id', 'groupName', 'removeHlsHdsFiles')
def remove_config(**kwargs):
    """
    This command will both stop the stream and remove the corresponding
    configuration entry. This command is the same as performing
    shutdownStream permanently=1.

    :param id: The configId of the configuration that needs to be removed.
        ConfigId's can be obtained from the listConfig interface. Removing an
        inbound stream will also automatically remove all associated outbound
        streams.
    :type id: int

    :param groupName: The name of the group that needs to be removed
        (applicable to HLS, HDS and external processes). Mandatory only if the
        id parameter is not specified.
    :type groupName: str

    :param removeHlsHdsFiles: If 1 (true) and the stream is HLS or HDS, the
        folder associated with it will be removed
    :type removeHlsHdsFiles: int

    :link: http://docs.evostream.com/ems_api_definition/removeconfig
    """
    return execute('removeconfig', **kwargs)



import json

from django.core.management.base import BaseCommand

from evostream import EvoStreamException


class EvoStreamCommand(BaseCommand):
    requires_system_checks = False

    def get_results(self, **options):
        raise NotImplementedError()

    def format_results(self, results, verbosity):
        self.stdout.write(json.dumps(results, indent=1, sort_keys=True))

    def handle(self, verbosity, *args, **options):
        try:
            results = self.get_results(**options)
        except EvoStreamException as ex:
            self.stderr.write(str(ex) + '\n')
            return
        if results is None:
            self.stdout.write('No data\n')
            return
        self.format_results(results, verbosity=verbosity)



from django.utils.functional import LazyObject

from .conf import settings
from .utils import get_module_class


class Protocol(LazyObject):
    def _setup(self):
        self._wrapped = get_module_class(settings.EVOSTREAM_PROTOCOL)()

protocol = Protocol()



from base64 import b64encode
import json
import socket
try:
    from httplib import HTTPConnection
except ImportError:
    from http.client import HTTPConnection

from .conf import settings
from . import EvoStreamException


class BaseProtocol(object):
    def get_result(self, command, **params):
        raise NotImplementedError()

    def parse_result(self, result):
        raise NotImplementedError()

    def execute(self, command, **params):
        result = self.get_result(command, **params)

        return self.parse_result(result)


class HTTPProtocol(BaseProtocol):
    @staticmethod
    def make_uri(command, **params):
        uri = '/%s' % command
        if len(params) > 0:
            str_params = ' '.join(['%s=%s' % (i, params[i]) for i in params])
            uri += '?params=%s' % b64encode(str_params)
        return uri

    def get_result(self, command, **params):
        conn = HTTPConnection(settings.EVOSTREAM_URL)
        uri = self.make_uri(command, **params)
        try:
            conn.request('GET', uri)
        except socket.error as ex:
            raise EvoStreamException(ex)
        response = conn.getresponse()
        return response.read()

    def parse_result(self, result):
        result = json.loads(result)
        if result['status'] == 'FAIL':
            raise EvoStreamException(result['description'])
        else:
            return result['data']



from codecs import open

from setuptools import setup, find_packages

from evostream import __version__, __author__, __email__, __license__

setup(
    name="django-evostream",
    version=__version__,
    author=__author__,
    author_email=__email__,
    url='https://github.com/tomi77/django-evostream',
    description='Manage EvoStream Media Server',
    long_description=open("README.md").read(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Multimedia :: Video',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
    ],
    license=__license__,
    packages=find_packages()
)



from django import get_version


VERSION = (0, 1, 0, 'beta', 2)
__author__ = "Tomasz Jakub Rup"
__email__ = "tomasz.rup@gmail.com"
__version__ = get_version(VERSION)
__license__ = "MIT"


class EvoStreamException(Exception):
    pass
